/***************************************************************************

New Super Pac-Man memory map (preliminary)

CPU #1:
0000-03ff video RAM
0400-07ff color RAM
0800-0f7f RAM
0f80-0fff sprite data 1 (sprite number & color)
1000-177f RAM
1780-17ff sprite data 2 (x, y position)
1800-1f7f RAM
1f80-1fff sprite data 3 (high bit of y, flip flags, double-size flags)
2000      watchdog timer?
4040-43ff RAM shared with CPU #2
4800-480f custom I/O chip #1
4810-481f custom I/O chip #2
5002-5003 IRQ enable
5008-5009 sound enable
500a-500b CPU #2 enable
8000      watchdog timer
c000-ffff ROM

CPU #2:
0000-0040 sound registers
0040-03ff RAM shared with CPU #1
f000-ffff ROM

Interrupts:
CPU #1 IRQ generated by VBLANK
CPU #2 uses no interrupts

***************************************************************************/

#include "driver.h"
#include "vidhrdw/generic.h"

extern unsigned char *mappy_soundregs;
void mappy_sound_enable_w(int offset,int data);
void mappy_sound_w(int offset,int data);
void mappy_sh_update(void);

extern unsigned char *superpac_sharedram;
extern unsigned char *superpac_customio_1,*superpac_customio_2;
int superpac_customio_r(int offset);
int superpac_sharedram_r(int offset);
int superpac_sharedram_r2(int offset);
void superpac_sharedram_w(int offset,int data);
void superpac_customio_w_1(int offset,int data);
void superpac_customio_w_2(int offset,int data);
int superpac_customio_r_1(int offset);
int superpac_customio_r_2(int offset);
void superpac_interrupt_enable_1_w(int offset,int data);
void superpac_cpu_enable_w(int offset,int data);
int superpac_interrupt_1(void);

int pacnpal_sharedram_r2(int offset);
void pacnpal_sharedram_w2(int offset,int data);
int pacnpal_interrupt_2(void);
void pacnpal_interrupt_enable_2_w(int offset,int data);

int superpac_vh_start(void);
void superpac_vh_screenrefresh(struct osd_bitmap *bitmap);
int superpac_init_machine(const char *gamename);
void superpac_vh_convert_color_prom(unsigned char *palette, unsigned char *colortable,const unsigned char *color_prom);


/* CPU 1 read addresses */
static struct MemoryReadAddress readmem_cpu1[] =
{
	{ 0x0000, 0x1fff, MRA_RAM },                                       /* general RAM */
	{ 0x4040, 0x43ff, superpac_sharedram_r, &superpac_sharedram },     /* shared RAM */
	{ 0x4800, 0x480f, superpac_customio_r_1, &superpac_customio_1 },   /* custom I/O chip #1 interface */
	{ 0x4810, 0x481f, superpac_customio_r_2, &superpac_customio_2 },   /* custom I/O chip #2 interface */
	{ 0xa000, 0xffff, MRA_ROM },                        /* SPC-2.1C at 0xc000, SPC-1.1B at 0xe000 */
	{ -1 }                                              /* end of table */
};


/* CPU 1 write addresses */
static struct MemoryWriteAddress writemem_cpu1[] =
{
	{ 0x0000, 0x03ff, videoram_w, &videoram, &videoram_size },          /* video RAM */
	{ 0x0400, 0x07ff, colorram_w, &colorram },          /* color RAM */
	{ 0x0800, 0x0f7f, MWA_RAM },                        /* RAM */
	{ 0x0f80, 0x0fff, MWA_RAM, &spriteram, &spriteram_size },            /* sprite RAM, area 1 */
	{ 0x1000, 0x177f, MWA_RAM },                        /* RAM */
	{ 0x1780, 0x17ff, MWA_RAM, &spriteram_2 },          /* sprite RAM, area 2 */
	{ 0x1800, 0x1f7f, MWA_RAM },                        /* RAM */
	{ 0x1f80, 0x1fff, MWA_RAM, &spriteram_3 },          /* sprite RAM, area 3 */
	{ 0x2000, 0x2000, MWA_NOP },                        /* watchdog timer */
	{ 0x4040, 0x43ff, MWA_RAM },                        /* shared RAM */
	{ 0x4800, 0x480f, superpac_customio_w_1 },          /* custom I/O chip #1 interface */
	{ 0x4810, 0x481f, superpac_customio_w_2 },          /* custom I/O chip #2 interface */
	{ 0x5002, 0x5003, superpac_interrupt_enable_1_w },  /* interrupt enable */
	{ 0x5008, 0x5009, mappy_sound_enable_w },           /* sound enable */
	{ 0x500a, 0x500b, superpac_cpu_enable_w },          /* interrupt enable */
	{ 0x8000, 0x8000, MWA_NOP },                        /* watchdog timer */
	{ 0xa000, 0xffff, MWA_ROM },                        /* SPC-2.1C at 0xc000, SPC-1.1B at 0xe000 */
	{ -1 }                                              /* end of table */
};


/* CPU 2 read addresses */
static struct MemoryReadAddress superpac_readmem_cpu2[] =
{
	{ 0xf000, 0xffff, MRA_ROM },                        /* ROM code */
	{ 0x0040, 0x03ff, superpac_sharedram_r2 },          /* shared RAM with the main CPU */
	{ -1 }                                              /* end of table */
};


/* CPU 2 write addresses */
static struct MemoryWriteAddress superpac_writemem_cpu2[] =
{
	{ 0x0040, 0x03ff, superpac_sharedram_w },           /* shared RAM with the main CPU */
	{ 0x0000, 0x003f, mappy_sound_w, &mappy_soundregs },/* sound control registers */
	{ 0xf000, 0xffff, MWA_ROM },                        /* ROM code */
	{ -1 }                                              /* end of table */
};


/* CPU 2 read addresses */
static struct MemoryReadAddress pacnpal_readmem_cpu2[] =
{
	{ 0xf000, 0xffff, MRA_ROM },                        /* ROM code */
	{ 0x0040, 0x03ff, pacnpal_sharedram_r2 },           /* shared RAM with the main CPU */
	{ -1 }                                              /* end of table */
};


/* CPU 2 write addresses */
static struct MemoryWriteAddress pacnpal_writemem_cpu2[] =
{
	{ 0x0040, 0x03ff, pacnpal_sharedram_w2 },           /* shared RAM with the main CPU */
	{ 0x0000, 0x003f, mappy_sound_w, &mappy_soundregs },/* sound control registers */
	{ 0x2000, 0x2001, pacnpal_interrupt_enable_2_w },   /* interrupt enable */
	{ 0x2006, 0x2007, mappy_sound_enable_w },           /* sound enable */
	{ 0xf000, 0xffff, MWA_ROM },                        /* ROM code */
	{ -1 }                                              /* end of table */
};


/* input ports & dip switches */
static struct InputPort superpac_input_ports[] =
{
	{	/* DSW1 */
		0x00,
		{ 0, 0, 0, 0, 0, 0, 0, 0 },
		{ 0, 0, 0, 0, 0, 0, 0, 0 }
	},
	{	/* DSW2 */
		0x00,
		{ 0, 0, 0, 0, 0, 0, 0, 0 },
		{ 0, 0, 0, 0, 0, 0, 0, 0 }
	},
	{	/* IN0 */
		0x00,
		{ OSD_KEY_UP, OSD_KEY_RIGHT, OSD_KEY_DOWN, OSD_KEY_LEFT, 0, OSD_KEY_CONTROL, 0, 0 },
		{ OSD_JOY_UP, OSD_JOY_RIGHT, OSD_JOY_DOWN, OSD_JOY_LEFT, 0, OSD_JOY_FIRE, 0, 0 },
	},
	{	/* IN1 */
		0x00,
		{ OSD_KEY_3, 0, 0, 0, OSD_KEY_1, OSD_KEY_2, 0, 0 },
		{ 0, 0, 0, 0, 0, 0, 0, 0 },
	},
	{ -1 }	/* end of table */
};


/* input ports & dip switches */
static struct InputPort pacnpal_input_ports[] =
{
	{	/* DSW1 */
		0x00,
		{ 0, 0, 0, 0, 0, 0, 0, 0 },
		{ 0, 0, 0, 0, 0, 0, 0, 0 }
	},
	{	/* DSW2 */
		0x98,
		{ 0, 0, 0, 0, 0, 0, 0, 0 },
		{ 0, 0, 0, 0, 0, 0, 0, 0 }
	},
	{	/* IN0 */
		0x00,
		{ OSD_KEY_UP, OSD_KEY_RIGHT, OSD_KEY_DOWN, OSD_KEY_LEFT, OSD_KEY_CONTROL, 0, 0, 0 },
		{ OSD_JOY_UP, OSD_JOY_RIGHT, OSD_JOY_DOWN, OSD_JOY_LEFT, OSD_JOY_FIRE, 0, 0, 0 },
	},
	{	/* IN1 */
		0x00,
		{ OSD_KEY_3, 0, 0, 0, OSD_KEY_1, OSD_KEY_2, 0, 0 },
		{ 0, 0, 0, 0, 0, 0, 0, 0 },
	},
	{ -1 }	/* end of table */
};


static struct TrakPort trak_ports[] =
{
	{ -1 }
};


/* keyboard info */
static struct KEYSet keys[] =
{
	{ 2, 0, "MOVE UP" },
	{ 2, 3, "MOVE LEFT"  },
	{ 2, 1, "MOVE RIGHT" },
	{ 2, 2, "MOVE DOWN" },
	{ 2, 5, "FIRE" },
	{ -1 }
};


/* dip switches... */
static struct DSW superpac_dsw[] =
{
	{ 0, 0x0F, "DIFFICULTY", { "R0 NORMAL","R1 EASIEST","R2","R3","R4","R5","R6","R7","R8 DEFAULT",\
								"R9","RA","RB HARDEST","RC EASIEST AUTO","RD","RE","RF HARDEST AUTO" } },
	{ 0, 0x40, "DEMO SOUND", { "ON", "OFF" }, 1 },
	{ 1, 0x07, "COINS", { "1 COIN 1 CREDIT", "1 COIN 2 CREDIT", "1 COIN 3 CREDIT",\
						 "1 COIN 6 CREDIT", "1 COIN 7 CREDIT", "2 COIN 1 CREDIT", \
						 "2 COIN 3 CREDIT", "3 COIN 1 CREDIT" } },
	{ 1, 0x38, "BONUS AT", { "30000 100000", "30000 80000", "30000 120000", "30000 80000@@@",\
							 "30000 100000@@@", "30000 120000@@@", "80000", "NONE" } },
	{ 1, 0xc0, "LIVES", { "3", "1", "2", "5" } },
	{ -1 }
};


/* dip switches... */
static struct DSW pacnpal_dsw[] =
{
	{ 0, 0x0c, "RANK", { "A", "B", "C", "D" } },
	{ 1, 0x07, "COINS", { "1 COIN 1 CREDIT", "1 COIN 2 CREDIT", "1 COIN 3 CREDIT",\
						 "1 COIN 6 CREDIT", "1 COIN 7 CREDIT", "2 COIN 1 CREDIT", \
						 "2 COIN 3 CREDIT", "3 COIN 1 CREDIT" } },
	{ 1, 0x38, "BONUS AT", { "NONE", "20K 70K 70K", "30K 30K 80K", "20K 70K", "30K 70K", "30K 80K", "30K 100K", "30K" } },
	{ 1, 0xc0, "LIVES", { "1", "2", "3", "5" } },
	{ -1 }
};


/* SUPERPAC -- ROM SPV-1.3C (4K) */
static struct GfxLayout charlayout =
{
	8,8,                                           /* 8*8 characters */
	256,                                           /* 256 characters */
	2,                                             /* 2 bits per pixel */
	{ 0, 4 },                                      /* the two bitplanes for 4 pixels are packed into one byte */
	{ 7*8, 6*8, 5*8, 4*8, 3*8, 2*8, 1*8, 0*8 },    /* characters are rotated 90 degrees */
	{ 8*8+0, 8*8+1, 8*8+2, 8*8+3, 0, 1, 2, 3 },    /* bits are packed in groups of four */
	16*8                                           /* every char takes 16 bytes */
};


/* SUPERPAC -- ROM SPV-2.3F (8K) */
static struct GfxLayout spritelayout =
{
	16,16,                                         /* 16*16 sprites */
	128,                                           /* 128 sprites */
	2,                                             /* 2 bits per pixel */
	{ 0, 4 },                                      /* the two bitplanes for 4 pixels are packed into one byte */
	{ 39 * 8, 38 * 8, 37 * 8, 36 * 8, 35 * 8, 34 * 8, 33 * 8, 32 * 8,
			7 * 8, 6 * 8, 5 * 8, 4 * 8, 3 * 8, 2 * 8, 1 * 8, 0 * 8 },
	{ 0, 1, 2, 3, 8*8, 8*8+1, 8*8+2, 8*8+3,
			16*8+0, 16*8+1, 16*8+2, 16*8+3, 24*8+0, 24*8+1, 24*8+2, 24*8+3 },
	64*8                                           /* every sprite takes 64 bytes */
};


static struct GfxDecodeInfo gfxdecodeinfo[] =
{
	{ 1, 0x0000, &charlayout,      0, 64 },
	{ 1, 0x1000, &spritelayout, 64*4, 64 },
	{ -1 } /* end of array */
};


static unsigned char superpac_color_prom[] =
{
	/* 4C - palette */
	0xf6,0xc9,0x3f,0x07,0xef,0xf8,0x2f,0xaf,0x3c,0x5d,0x38,0xe7,0x29,0x66,0x54,0x00,
	0x00,0xd8,0x66,0x29,0xe7,0x38,0x5d,0xd5,0xaf,0x2f,0xb8,0xef,0x07,0x3f,0xf6,0x00,
	/* 4E - chars */
	0x00,0x00,0x00,0x00,0x00,0x02,0x02,0x00,0x00,0x03,0x01,0x00,0x00,0x0b,0x03,0x0e,
	0x00,0x0b,0x01,0x0e,0x00,0x06,0x01,0x0b,0x00,0x0b,0x01,0x08,0x00,0x04,0x01,0x07,
	0x00,0x04,0x01,0x05,0x00,0x02,0x06,0x04,0x00,0x02,0x06,0x03,0x00,0x01,0x01,0x01,
	0x00,0x03,0x01,0x0d,0x00,0x02,0x06,0x0d,0x00,0x0d,0x09,0x01,0x00,0x09,0x07,0x0a,
	0x00,0x01,0x06,0x09,0x00,0x02,0x09,0x0a,0x00,0x03,0x0b,0x0a,0x00,0x01,0x07,0x03,
	0x00,0x04,0x0b,0x0a,0x00,0x0a,0x01,0x0c,0x00,0x01,0x04,0x03,0x00,0x06,0x09,0x01,
	0x00,0x02,0x06,0x01,0x00,0x0a,0x01,0x0c,0x00,0x03,0x02,0x0e,0x00,0x01,0x0b,0x03,
	0x00,0x00,0x03,0x00,0x00,0x00,0x04,0x00,0x00,0x0a,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	/* 3L - sprites */
	0x0f,0x0f,0x0f,0x0f,0x0f,0x02,0x02,0x02,0x0f,0x00,0x00,0x00,0x0f,0x00,0x04,0x05,
	0x0f,0x03,0x00,0x01,0x0f,0x04,0x00,0x01,0x0f,0x05,0x00,0x01,0x0f,0x06,0x00,0x01,
	0x0f,0x01,0x04,0x00,0x00,0x00,0x00,0x00,0x0f,0x0f,0x00,0x01,0x0f,0x07,0x00,0x00,
	0x0f,0x0f,0x0f,0x0f,0x0f,0x08,0x0f,0x0f,0x0f,0x08,0x08,0x0f,0x0f,0x08,0x08,0x08,
	0x0f,0x03,0x00,0x0d,0x0f,0x02,0x06,0x0d,0x0f,0x0d,0x0e,0x00,0x0f,0x09,0x07,0x0a,
	0x0f,0x00,0x06,0x09,0x0f,0x02,0x09,0x0a,0x0f,0x03,0x0b,0x08,0x0f,0x00,0x07,0x03,
	0x0f,0x04,0x0b,0x0a,0x0f,0x0a,0x0c,0x00,0x0f,0x00,0x04,0x03,0x0f,0x06,0x09,0x00,
	0x0f,0x02,0x00,0x06,0x0f,0x0a,0x00,0x0c,0x0f,0x03,0x02,0x01,0x0f,0x00,0x0b,0x03,
	0x0f,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x0f,0x00,0x00,0x00,0x0f,0x00,0x00,0x00,
	0x0f,0x05,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x0f,0x02,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x0f,0x02,0x0f,0x0f,0x0f,0x02,0x02,0x0f,0x0f,0x02,0x02,0x02,0x0f,0x02,0x02,0x0f,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x0f,0x00,0x04,0x00,0x0f,0x00,0x05,0x00,0x0f,0x00,0x06,0x00,0x0f,0x00,0x08,0x00
};


static unsigned char pacnpal_color_prom[] =
{
	/* 4C - palette */
	0xf6,0xc0,0x3f,0x3c,0x8f,0x07,0xe7,0xf0,0x2f,0x28,0xc9,0xed,0x5d,0xa4,0xf6,0x00,
	0x00,0xf6,0x5d,0xc9,0xe0,0xf0,0xba,0x38,0x3c,0x3f,0x27,0xb7,0xef,0x8f,0x07,0x00,
	/* 4E - chars */
	0x00,0x06,0x05,0x01,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x01,0x0e,0x00,
	0x00,0x07,0x0e,0x02,0x00,0x01,0x0e,0x0d,0x00,0x06,0x0e,0x02,0x00,0x00,0x00,0x00,
	0x00,0x03,0x00,0x04,0x05,0x03,0x00,0x04,0x00,0x0b,0x00,0x09,0x0c,0x0b,0x00,0x09,
	0x00,0x02,0x00,0x03,0x00,0x0b,0x00,0x0d,0x00,0x0d,0x00,0x0e,0x00,0x09,0x00,0x0c,
	0x00,0x00,0x0e,0x00,0x00,0x00,0x01,0x00,0x00,0x00,0x02,0x00,0x00,0x00,0x03,0x00,
	0x00,0x00,0x04,0x00,0x00,0x00,0x05,0x00,0x00,0x00,0x06,0x00,0x00,0x00,0x07,0x00,
	0x00,0x00,0x08,0x00,0x00,0x00,0x09,0x00,0x00,0x00,0x0d,0x00,0x00,0x00,0x0b,0x00,
	0x00,0x00,0x0c,0x00,0x00,0x00,0x0d,0x00,0x0f,0x0f,0x0f,0x0f,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	/* 3L - sprites */
	0x0f,0x0f,0x0f,0x0f,0x0f,0x02,0x0f,0x0f,0x0f,0x03,0x0e,0x04,0x0f,0x05,0x00,0x01,
	0x0f,0x06,0x00,0x01,0x0f,0x07,0x00,0x01,0x0f,0x08,0x00,0x01,0x0f,0x01,0x07,0x0e,
	0x0f,0x0d,0x00,0x0b,0x0f,0x04,0x00,0x0d,0x00,0x00,0x00,0x00,0x0f,0x0d,0x00,0x02,
	0x0f,0x00,0x00,0x00,0x0f,0x07,0x07,0x00,0x0f,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x0f,0x05,0x02,0x0d,0x0f,0x05,0x0b,0x0e,0x0f,0x02,0x08,0x0e,0x0f,0x0e,0x05,0x08,
	0x00,0x00,0x00,0x00,0x0f,0x0e,0x05,0x05,0x0f,0x05,0x00,0x01,0x00,0x00,0x00,0x00,
	0x0f,0x05,0x0e,0x0c,0x0f,0x05,0x0e,0x03,0x0f,0x08,0x03,0x09,0x0f,0x03,0x0e,0x09,
	0x0f,0x02,0x0e,0x09,0x0f,0x08,0x0c,0x03,0x0f,0x07,0x0d,0x0e,0x00,0x00,0x00,0x00,
	0x0f,0x0e,0x05,0x0d,0x0f,0x0b,0x05,0x0c,0x0f,0x0b,0x05,0x09,0x0f,0x0b,0x08,0x03,
	0x0f,0x0b,0x09,0x03,0x0f,0x0b,0x02,0x03,0x0f,0x0b,0x08,0x03,0x0f,0x0b,0x0d,0x0d,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x0f,0x07,0x0b,0x0e,0x0f,0x0e,0x0d,0x0b,0x0f,0x0e,0x0b,0x07,0x0f,0x02,0x0f,0x0e,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x00,0x0f,0x0f,0x0f,0x00,0x00,
	0x0f,0x00,0x0f,0x0f,0x0f,0x00,0x00,0x0f,0x0f,0x0f,0x0f,0x00,0x0f,0x00,0x00,0x00
};


/* waveforms for the audio hardware */
static unsigned char superpac_samples[8*32] =
{
	0xf8,0x18,0x28,0x38,0x48,0x58,0x58,0x68,0x68,0x68,0x58,0x58,0x48,0x38,0x28,0x18,
	0xf8,0xd8,0xc8,0xb8,0xa8,0x98,0x98,0x88,0x88,0x88,0x98,0x98,0xa8,0xb8,0xc8,0xd8,

	0x88,0x98,0xa8,0xb8,0xc8,0xd8,0xe8,0xf8,0x08,0x18,0x28,0x38,0x48,0x58,0x68,0x78,
	0x78,0x68,0x58,0x48,0x38,0x28,0x18,0x08,0xf8,0xe8,0xd8,0xc8,0xb8,0xa8,0x98,0x88,

	0xf8,0x08,0xe8,0x18,0xd8,0x28,0xc8,0x38,0xb8,0x48,0xa8,0x58,0x98,0x68,0x88,0x78,
	0x88,0x78,0x98,0x68,0xa8,0x58,0xb8,0x48,0xc8,0x38,0xd8,0x28,0xe8,0x18,0xf8,0x08,

	0xf8,0x28,0x48,0x58,0x68,0x58,0x48,0x28,0xf8,0xc8,0xa8,0x98,0x88,0x98,0xa8,0xc8,
	0xf8,0x38,0x58,0x68,0x58,0x38,0xf8,0xb8,0x98,0x88,0x98,0xb8,0xf8,0x68,0xf8,0x88,

	0x68,0x68,0x68,0x68,0x68,0x68,0x68,0x68,0x68,0x68,0x68,0x68,0x68,0x68,0x68,0x68,
	0x88,0x88,0x88,0x88,0x88,0x88,0x88,0x88,0x88,0x88,0x88,0x88,0x88,0x88,0x88,0x88,

	0x68,0x48,0xf8,0x28,0x48,0xf8,0xd8,0x48,0x68,0x58,0x08,0x28,0x38,0xe8,0xa8,0xf8,
	0x48,0x08,0xb8,0xc8,0xe8,0x98,0x88,0xb8,0x18,0xf8,0xa8,0xc8,0xf8,0xa8,0x88,0xf8,

	0xf8,0x48,0x68,0x68,0x58,0x38,0x18,0x28,0x38,0x38,0x28,0x18,0xe8,0xc8,0xb8,0xd8,
	0xf8,0x18,0x38,0x28,0x08,0xd8,0xc8,0xb8,0xb8,0xc8,0xd8,0xb8,0x98,0x88,0x88,0xa8,

	0x48,0x68,0x68,0x68,0x68,0x48,0x38,0x18,0xf8,0xe8,0xd8,0xd8,0xd8,0xe8,0x08,0x28,
	0x38,0x38,0x18,0xe8,0xc8,0xa8,0xa8,0xa8,0xb8,0xd8,0xc8,0xb8,0xa8,0x98,0xb8,0xf8
};

static unsigned char pacnpal_samples[8*32] =
{
	0xf8,0x58,0x68,0x78,0x48,0x58,0x58,0x68,0x68,0x68,0x58,0x58,0x48,0x78,0x68,0x58,
	0xf8,0xd8,0xc8,0xf8,0xe8,0xd8,0xd8,0xc8,0xc8,0xc8,0xd8,0xd8,0xe8,0xf8,0xc8,0xd8,

	0x48,0x68,0x68,0x68,0x68,0x58,0x58,0x68,0x68,0x58,0x68,0x58,0x48,0xe8,0xe8,0xf8,
	0x48,0x48,0xe8,0xd8,0xc8,0xd8,0xc8,0xc8,0xd8,0xd8,0xc8,0xc8,0xc8,0xc8,0xe8,0xf8,

	0x48,0x68,0x68,0x68,0x68,0x58,0x78,0x48,0x68,0x68,0x48,0x68,0x48,0x68,0xe8,0xd8,
	0x48,0x48,0xe8,0xc8,0xe8,0xd8,0xc8,0xc8,0xf8,0xd8,0xc8,0xc8,0xc8,0xc8,0xe8,0xd8,

	0xf8,0x68,0x48,0x58,0x68,0x58,0x48,0x68,0xf8,0xc8,0xe8,0xd8,0xc8,0xd8,0xe8,0xc8,
	0xf8,0x78,0x58,0x68,0x58,0x78,0xf8,0xf8,0xd8,0xc8,0xd8,0xf8,0xf8,0x68,0xf8,0xc8,

	0x68,0x68,0x68,0x68,0x68,0x68,0x68,0x68,0x68,0x68,0x68,0x68,0x68,0x68,0x68,0x68,
	0xc8,0xc8,0xc8,0xc8,0xc8,0xc8,0xc8,0xc8,0xc8,0xc8,0xc8,0xc8,0xc8,0xc8,0xc8,0xc8,

	0x68,0x48,0xf8,0x68,0x48,0xf8,0xd8,0x48,0x68,0x58,0x48,0x68,0x78,0xe8,0xe8,0xf8,
	0x48,0x48,0xf8,0xc8,0xe8,0xd8,0xc8,0xf8,0x58,0xf8,0xe8,0xc8,0xf8,0xe8,0xc8,0xf8,

	0x68,0x68,0x68,0x68,0x68,0x68,0x48,0x58,0x48,0x68,0xf8,0x48,0xd8,0xf8,0xd8,0xd8,
	0x48,0x78,0x68,0x68,0x48,0xc8,0xe8,0xd8,0xd8,0xc8,0xd8,0xf8,0xd8,0xc8,0xd8,0xf8,

	0x48,0x68,0x68,0x68,0x68,0x48,0x78,0x58,0xf8,0xe8,0xd8,0xd8,0xd8,0xe8,0x48,0x68,
	0x78,0x78,0x58,0xe8,0xc8,0xe8,0xe8,0xe8,0xf8,0xd8,0xc8,0xb8,0xa8,0x98,0xb8,0xf8
};


static struct MachineDriver superpac_machine_driver =
{
	/* basic machine hardware  */
	{
		{
			CPU_M6809,
			1100000,             /* 1.1 Mhz */
			0,                   /* memory region */
			readmem_cpu1,        /* MemoryReadAddress */
			writemem_cpu1,       /* MemoryWriteAddress */
			0,                   /* IOReadPort */
			0,                   /* IOWritePort */
			interrupt,           /* interrupt routine */
			1                    /* interrupts per frame */
		},
		{
			CPU_M6809,
			1100000,             /* 1.1 Mhz */
			2,                   /* memory region */
			superpac_readmem_cpu2,/* MemoryReadAddress */
			superpac_writemem_cpu2,/* MemoryWriteAddress */
			0,                   /* IOReadPort */
			0,                   /* IOWritePort */
			ignore_interrupt,		/* interrupt routine */
			1                    /* interrupts per frame */
		}
	},
	60,                        /* frames per second */
	superpac_init_machine,     /* init machine routine */

	/* video hardware */
	28*8, 36*8,                /* screen_width, screen_height */
	{ 0*8, 28*8-1, 0*8, 36*8-1 },/* struct rectangle visible_area */
	gfxdecodeinfo,             /* GfxDecodeInfo * */
	32,                        /* total colors */
	4*(64+64),                 /* color table length */
	superpac_vh_convert_color_prom, /* convert color prom routine */

	0,                         /* vh_init routine */
	generic_vh_start,          /* vh_start routine */
	generic_vh_stop,           /* vh_stop routine */
	superpac_vh_screenrefresh, /* vh_update routine */

	/* sound hardware */
	superpac_samples,          /* pointer to samples */
	0,                         /* sh_init routine */
	0,                         /* sh_start routine */
	0,                         /* sh_stop routine */
	mappy_sh_update            /* sh_update routine */
};

static struct MachineDriver pacnpal_machine_driver =
{
	/* basic machine hardware  */
	{
		{
			CPU_M6809,
			1100000,             /* 1.1 Mhz */
			0,                   /* memory region */
			readmem_cpu1,        /* MemoryReadAddress */
			writemem_cpu1,       /* MemoryWriteAddress */
			0,                   /* IOReadPort */
			0,                   /* IOWritePort */
			interrupt,           /* interrupt routine */
			1                    /* interrupts per frame */
		},
		{
			CPU_M6809,
			1100000,             /* 1.1 Mhz */
			2,                   /* memory region */
			pacnpal_readmem_cpu2,/* MemoryReadAddress */
			pacnpal_writemem_cpu2,/* MemoryWriteAddress */
			0,                   /* IOReadPort */
			0,                   /* IOWritePort */
			pacnpal_interrupt_2, /* interrupt routine */
			1                    /* interrupts per frame */
		}
	},
	60,                        /* frames per second */
	superpac_init_machine,     /* init machine routine */

	/* video hardware */
	28*8, 36*8,                /* screen_width, screen_height */
	{ 0*8, 28*8-1, 0*8, 36*8-1 },/* struct rectangle visible_area */
	gfxdecodeinfo,             /* GfxDecodeInfo * */
	32,                        /* total colors */
	4*(64+64),                 /* color table length */
	superpac_vh_convert_color_prom, /* convert color prom routine */

	0,                         /* vh_init routine */
	generic_vh_start,          /* vh_start routine */
	generic_vh_stop,           /* vh_stop routine */
	superpac_vh_screenrefresh, /* vh_update routine */

	/* sound hardware */
	pacnpal_samples,            /* pointer to samples */
	0,                         /* sh_init routine */
	0,                         /* sh_start routine */
	0,                         /* sh_stop routine */
	mappy_sh_update            /* sh_update routine */
};


ROM_START( superpac_rom )
	ROM_REGION(0x10000)	/* 64k for code */
	ROM_LOAD( "SPC-2.1C", 0xc000, 0x2000, 0x8da57a9b )
	ROM_LOAD( "SPC-1.1B", 0xe000, 0x2000, 0xcb391cdf )

	ROM_REGION(0x3000)	/* temporary space for graphics (disposed after conversion) */
	ROM_LOAD( "SPV-1.3C", 0x0000, 0x1000, 0xac21e9f9 )
	ROM_LOAD( "SPV-2.3F", 0x1000, 0x2000, 0x19a3fd43 )

	ROM_REGION(0x10000)	/* 64k for the second CPU */
	ROM_LOAD( "SPC-3.1K", 0xf000, 0x1000, 0x6d32e9fa )
ROM_END



ROM_START( pacnpal_rom )
	ROM_REGION(0x10000)	/* 64k for code */
	ROM_LOAD( "pap13b.cpu", 0xa000, 0x2000, 0x3c456471 )
	ROM_LOAD( "pap12b.cpu", 0xc000, 0x2000, 0x773e919c )
	ROM_LOAD( "pap11b.cpu", 0xe000, 0x2000, 0x30f54a3d )

	ROM_REGION(0x3000)	/* temporary space for graphics (disposed after conversion) */
	ROM_LOAD( "pap16.cpu", 0x0000, 0x1000, 0x9565c97b )
	ROM_LOAD( "pap15.vid", 0x1000, 0x2000, 0x5a90f48c )

	ROM_REGION(0x10000)	/* 64k for the second CPU */
	ROM_LOAD( "pap14.cpu", 0xf000, 0x1000, 0xcda756c1 )
ROM_END


/* load the high score table */
static int superpac_hiload(const char *name)
{
   int writing = 0;
   FILE *f;

   /* get RAM pointer (this game is multiCPU, we can't assume the global */
   /* RAM pointer is pointing to the right place) */
   unsigned char *RAM = Machine->memory_region[0];

   /* check if the hi score table has already been initialized */
   if (memcmp(&RAM[0x113c],"N@N",3) == 0 &&        /* check for high score initials */
       RAM[0x1087] == 0 && RAM[0x1089] == 0 && RAM[0x1088] != 0 && /* check for main high score value */
   	 memcmp(&RAM[0x3ee],"000",3) == 0)           /* see if main high score was written to screen */
   {
      if ((f = fopen(name,"rb")) != 0)
      {
         fread(&RAM[0x1138],1,40,f);
         fclose(f);

         /* also copy over the high score */
         RAM[0x1087] = RAM[0x1138];
         RAM[0x1088] = RAM[0x1139];
         RAM[0x1089] = RAM[0x113a];
      }

      /* this is a little gross, but necessary to get the high score on-screen */
      if (!writing) writing = (RAM[0x1087] >> 4);
      videoram_w (0x3f4, writing ? (RAM[0x1087] >> 4) + '0' : ' ');
      if (!writing) writing = (RAM[0x1087] & 0x0f);
      videoram_w (0x3f3, writing ? (RAM[0x1087] & 0x0f) + '0' : ' ');
      if (!writing) writing = (RAM[0x1088] >> 4);
      videoram_w (0x3f2, writing ? (RAM[0x1088] >> 4) + '0' : ' ');
      if (!writing) writing = (RAM[0x1088] & 0x0f);
      videoram_w (0x3f1, writing ? (RAM[0x1088] & 0x0f) + '0' : ' ');
      if (!writing) writing = (RAM[0x1089] >> 4);
      videoram_w (0x3f0, writing ? (RAM[0x1089] >> 4) + '0' : ' ');
      if (!writing) writing = (RAM[0x1089] & 0x0f);
      videoram_w (0x3ef, writing ? (RAM[0x1089] & 0x0f) + '0' : ' ');
      videoram_w (0x3ee, 0 + '0');

      return 1;
   }
   else return 0; /* we can't load the hi scores yet */
}


/* save the high score table */
static void superpac_hisave(const char *name)
{
   FILE *f;

   /* get RAM pointer (this game is multiCPU, we can't assume the global */
   /* RAM pointer is pointing to the right place) */
   unsigned char *RAM = Machine->memory_region[0];

   if ((f = fopen(name,"wb")) != 0)
   {
      fwrite(&RAM[0x1138],1,40,f);
      fclose(f);
   }
}


/* load the high score table */
static int pacnpal_hiload(const char *name)
{
   int writing = 0;
   FILE *f;

   /* get RAM pointer (this game is multiCPU, we can't assume the global */
   /* RAM pointer is pointing to the right place) */
   unsigned char *RAM = Machine->memory_region[0];

   /* check if the hi score table has already been initialized */
   if (memcmp(&RAM[0x1051],"\x1a\x25\x1a",3) == 0 &&        /* check for high score initials */
       RAM[0x116d] == 0 && RAM[0x116f] == 0 && RAM[0x116e] != 0 && /* check for main high score value */
   	 memcmp(&RAM[0x3ed],"\x0\x0\x0",3) == 0)           /* see if main high score was written to screen */
   {
      if ((f = fopen(name,"rb")) != 0)
      {
         fread(&RAM[0x104c],1,40,f);
         fclose(f);

         /* also copy over the high score */
         RAM[0x116d] = RAM[0x104d];
         RAM[0x116e] = RAM[0x104e];
         RAM[0x116f] = RAM[0x104f];
      }

      /* this is a little gross, but necessary to get the high score on-screen */
      if (!writing) writing = (RAM[0x116d] >> 4);
      videoram_w (0x3f3, writing ? (RAM[0x116d] >> 4) : 0x24);
      if (!writing) writing = (RAM[0x116d] & 0x0f);
      videoram_w (0x3f2, writing ? (RAM[0x116d] & 0x0f) : 0x24);
      if (!writing) writing = (RAM[0x116e] >> 4);
      videoram_w (0x3f1, writing ? (RAM[0x116e] >> 4) : 0x24);
      if (!writing) writing = (RAM[0x116e] & 0x0f);
      videoram_w (0x3f0, writing ? (RAM[0x116e] & 0x0f) : 0x24);
      if (!writing) writing = (RAM[0x116f] >> 4);
      videoram_w (0x3ef, writing ? (RAM[0x116f] >> 4) : 0x24);
      if (!writing) writing = (RAM[0x116f] & 0x0f);
      videoram_w (0x3ee, writing ? (RAM[0x116f] & 0x0f) : 0x24);
      videoram_w (0x3ed, 0);

      return 1;
   }
   else return 0; /* we can't load the hi scores yet */
}


/* save the high score table */
static void pacnpal_hisave(const char *name)
{
   FILE *f;

   /* get RAM pointer (this game is multiCPU, we can't assume the global */
   /* RAM pointer is pointing to the right place) */
   unsigned char *RAM = Machine->memory_region[0];

   if ((f = fopen(name,"wb")) != 0)
   {
      fwrite(&RAM[0x104c],1,40,f);
      fclose(f);
   }
}


struct GameDriver superpac_driver =
{
	"Super Pac-Man",
	"superpac",
	"AARON GILES\nKEVIN BRISLEY\nALAN J MCCORMICK",
	&superpac_machine_driver, /* MachineDriver * */

	superpac_rom,             /* RomModule * */
	0, 0,                     /* ROM decrypt routines */
	0,                        /* samplenames */

	superpac_input_ports,     /* InputPort  */
	trak_ports,
	superpac_dsw,             /* DSW        */
        keys,                     /* KEY def    */

	superpac_color_prom,		  /* color prom */
	0,                        /* palette */
	0,                        /* color table */

	8*11, 8*20,               /* paused_x, paused_y, paused_color for PAUSED */

	superpac_hiload, superpac_hisave /* hi score save/load */
};

struct GameDriver pacnpal_driver =
{
	"Pac & Pal",
	"pacnpal",
	"AARON GILES\nKEVIN BRISLEY\nLAWNMOWER MAN",
	&pacnpal_machine_driver,  /* MachineDriver * */

	pacnpal_rom,              /* RomModule * */
	0, 0,                     /* ROM decrypt routines */
	0,                        /* samplenames */

	pacnpal_input_ports,      /* InputPort  */
	trak_ports,
	pacnpal_dsw,              /* DSW        */
        keys,                     /* KEY def    */

	pacnpal_color_prom,       /* color prom */
	0,                        /* palette */
	0,                        /* color table */

	8*11, 8*20,               /* paused_x, paused_y, paused_color for PAUSED */

	pacnpal_hiload, pacnpal_hisave /* hi score save/load */
};


